Package: CellWindX
Title: Marker Gene Analysis and Visualization for Single-Cell Data
Version: 1.0.0
Authors@R: 
    c(
        person(
            given = "Xiaofeng",
            family = "Yang",
            email = "Youngxf02@163.com",
            role = c("aut", "cre"),
            comment = c(affiliation = "Chongqing Medical University")
        ),
        person(
            given = "Shan",
            family = "Li",
            email = "andylee24@foxmail.com",
            role = "aut",
            comment = c(affiliation = "Chongqing Medical University")
        )
     )
Description: Provides a 'Seurat'-compatible toolkit for marker gene identification,
    expression summarization, and visualization of annotated single-cell
    transcriptomic data. 'CellWindX' identifies top cell-type-enriched markers,
    calculates marker expression percentages and average expression values across
    cell groups, and generates publication-oriented dimensional reduction plots,
    marker heatmaps, and gene-level radar plots. The package includes built-in
    aesthetic palettes and supports both exploratory analysis and downstream figure
    preparation for single-cell atlas studies. The workflow is designed to
    complement single-cell analysis frameworks such as 'Seurat' described by
    Satija et al. (2015) <doi:10.1038/nbt.3192> and Hao et al. (2021)
    <doi:10.1016/j.cell.2021.04.048>, as well as heatmap visualization methods
    implemented in 'ComplexHeatmap' described by Gu et al. (2016)
    <doi:10.1093/bioinformatics/btw313>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: circlize, ComplexHeatmap, dplyr, ggplot2, grDevices, grid,
        Matrix, patchwork, Seurat, tidyr, stats
Suggests: SeuratObject
NeedsCompilation: no
Packaged: 2026-05-20 13:36:35 UTC; young
Author: Xiaofeng Yang [aut, cre] (affiliation: Chongqing Medical University),
  Shan Li [aut] (affiliation: Chongqing Medical University)
Maintainer: Xiaofeng Yang <Youngxf02@163.com>
Depends: R (>= 4.1.0)
Repository: CRAN
Date/Publication: 2026-05-27 19:50:27 UTC
Built: R 4.6.0; ; 2026-05-27 21:47:07 UTC; unix

Package: CellWindX
Title: Marker Gene Analysis and Visualization for Single-Cell Data
Version: 1.0.0
Authors@R: 
    c(
        person(
            given = "Xiaofeng",
            family = "Yang",
            email = "Youngxf02@163.com",
            role = c("aut", "cre"),
            comment = c(affiliation = "Chongqing Medical University")
        ),
        person(
            given = "Shan",
            family = "Li",
            email = "andylee24@foxmail.com",
            role = "aut",
            comment = c(affiliation = "Chongqing Medical University")
        )
     )
Description: Provides a 'Seurat'-compatible toolkit for marker gene identification,
    expression summarization, and visualization of annotated single-cell
    transcriptomic data. 'CellWindX' identifies top cell-type-enriched markers,
    calculates marker expression percentages and average expression values across
    cell groups, and generates publication-oriented dimensional reduction plots,
    marker heatmaps, and gene-level radar plots. The package includes built-in
    aesthetic palettes and supports both exploratory analysis and downstream figure
    preparation for single-cell atlas studies. The workflow is designed to
    complement single-cell analysis frameworks such as 'Seurat' described by
    Satija et al. (2015) <doi:10.1038/nbt.3192> and Hao et al. (2021)
    <doi:10.1016/j.cell.2021.04.048>, as well as heatmap visualization methods
    implemented in 'ComplexHeatmap' described by Gu et al. (2016)
    <doi:10.1093/bioinformatics/btw313>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: circlize, ComplexHeatmap, dplyr, ggplot2, grDevices, grid,
        Matrix, patchwork, Seurat, tidyr, stats
Suggests: SeuratObject
NeedsCompilation: no
Packaged: 2026-05-20 13:36:35 UTC; young
Author: Xiaofeng Yang [aut, cre] (affiliation: Chongqing Medical University),
  Shan Li [aut] (affiliation: Chongqing Medical University)
Maintainer: Xiaofeng Yang <Youngxf02@163.com>
Depends: R (>= 4.1.0)
Repository: CRAN
Date/Publication: 2026-05-27 19:50:27 UTC
Built: R 4.5.2; ; 2026-05-27 21:27:44 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CellWindX_DimPlot)
export(CellWindX_GeneRadar)
export(CellWindX_MarkerHeatmap)
export(CellWindX_TopMarkersStats)
importFrom(stats,ave)
importFrom(stats,median)
importFrom(stats,setNames)
importFrom(utils,write.csv)

#' Draw marker-gene heatmap across annotated cell groups
#'
#' @description
#' `CellWindX_MarkerHeatmap()` visualizes marker gene expression statistics
#' across annotated cell groups using either `ComplexHeatmap` or `ggplot2`.
#'
#' @param marker_result A result object returned by `CellWindX_TopMarkersStats()`,
#'   or a data frame containing marker expression statistics.
#' @param value_col Character string. Value column used for heatmap visualization.
#'   One of `"avg_expr"`, `"pct_expr"`, or `"avg_expr_positive"`.
#' @param scale_method Character string. Scaling method. One of `"zscore"` or `"none"`.
#' @param plot_engine Character string. Plotting engine. One of `"complex"` or `"ggplot"`.
#' @param palette Character string. Built-in CellWindX palette.
#' @param cluster_rows Logical. Whether to cluster rows.
#' @param cluster_columns Logical. Whether to cluster columns.
#' @param show_row_names Logical. Whether to show row names.
#' @param show_column_names Logical. Whether to show column names.
#' @param column_names_rot Numeric. Rotation angle of column names.
#' @param round_cell Logical. Whether to draw rounded cells when using ComplexHeatmap.
#' @param cell_width_mm Numeric. Cell width in millimeters.
#' @param cell_height_mm Numeric. Cell height in millimeters.
#' @param row_fontsize Numeric. Row name font size.
#' @param column_fontsize Numeric. Column name font size.
#' @param legend_position Character string. Legend position.
#' @param heatmap_title Character string or NULL. Heatmap title.
#' @param zscore_clip Numeric. Z-score clipping threshold.
#' @param output_file Character string or NULL. Optional output file path.
#' @param output_width Numeric. Output width in inches.
#' @param output_height Numeric. Output height in inches.
#' @param dpi Numeric. Output resolution.
#' @param draw Logical. Whether to draw the heatmap immediately when using ComplexHeatmap.
#'
#' @return A list with class `"CellWindX_MarkerHeatmap"`.
#'
#' @export
CellWindX_MarkerHeatmap <- function(
    marker_result,
    value_col = c("avg_expr", "pct_expr", "avg_expr_positive"),
    scale_method = c("zscore", "none"),
    plot_engine = c("complex", "ggplot"),
    palette = c("shanshui", "chongqing_modern", "girlish"),
    cluster_rows = FALSE,
    cluster_columns = FALSE,
    show_row_names = TRUE,
    show_column_names = TRUE,
    column_names_rot = 45,
    round_cell = TRUE,
    cell_width_mm = 5.5,
    cell_height_mm = 5.5,
    row_fontsize = 10,
    column_fontsize = 9,
    legend_position = "right",
    heatmap_title = NULL,
    zscore_clip = 2,
    output_file = NULL,
    output_width = 9,
    output_height = 5,
    dpi = 300,
    draw = TRUE
) {

  # -----------------------------
  # 1. Package check
  # -----------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.")
  }
  if (!requireNamespace("dplyr", quietly = TRUE)) {
    stop("Package 'dplyr' is required.")
  }
  if (!requireNamespace("tidyr", quietly = TRUE)) {
    stop("Package 'tidyr' is required.")
  }

  value_col <- match.arg(value_col)
  scale_method <- match.arg(scale_method)
  plot_engine <- match.arg(plot_engine)
  palette <- match.arg(palette)

  if (plot_engine == "complex") {
    if (!requireNamespace("ComplexHeatmap", quietly = TRUE)) {
      stop("Package 'ComplexHeatmap' is required for plot_engine = 'complex'.")
    }
    if (!requireNamespace("circlize", quietly = TRUE)) {
      stop("Package 'circlize' is required for plot_engine = 'complex'.")
    }
    if (!requireNamespace("grid", quietly = TRUE)) {
      stop("Package 'grid' is required for plot_engine = 'complex'.")
    }
  }

  # -----------------------------
  # 2. Built-in CellWindX palettes
  # -----------------------------
  CellWindX_palettes <- list(

    shanshui = list(
      discrete = c(
        "#8FB9A8", "#B7C8A3", "#A7C7D9", "#7FA6C7", "#9D9AC7",
        "#C6B7D9", "#D8C7A3", "#BFA58A", "#8A9A8F", "#6F8F9F",
        "#A6B1C2", "#C9D6DF", "#D6C6B9", "#E2D8C3", "#9E8FB2",
        "#738A99", "#B5A789", "#AFC8BA", "#D1A7A0", "#6E7F80"
      ),
      heatmap_low = "#6F8F9F",
      heatmap_mid = "#F7F4EA",
      heatmap_high = "#B58C9E"
    ),

    chongqing_modern = list(
      discrete = c(
        "#2F4858", "#33658A", "#4A90A4", "#86BBD8", "#758E4F",
        "#F6AE2D", "#F26419", "#D64550", "#8E5572", "#5D5179",
        "#3A506B", "#1C7293", "#00A6A6", "#7D8491", "#B8B8D1",
        "#E0A458", "#C8553D", "#6D597A", "#355070", "#2A2D34"
      ),
      heatmap_low = "#355070",
      heatmap_mid = "#F7F7F7",
      heatmap_high = "#F26419"
    ),

    girlish = list(
      discrete = c(
        "#FF9AA2", "#FFB7B2", "#FFDAC1", "#E2F0CB", "#B5EAD7",
        "#C7CEEA", "#F6C1D5", "#D5AAFF", "#A0E7E5", "#B4F8C8",
        "#FBE7C6", "#F7A8B8", "#F3C4FB", "#BFD7FF", "#FFD6E0",
        "#FFABAB", "#FFC3A0", "#D4F0F0", "#CCE2CB", "#E7C6FF"
      ),
      heatmap_low = "#A0C4FF",
      heatmap_mid = "#FFFFFF",
      heatmap_high = "#FF8FAB"
    )
  )

  pal_use <- CellWindX_palettes[[palette]]

  # -----------------------------
  # 3. Extract input data
  # -----------------------------
  if (is.list(marker_result) && "marker_expr_by_group" %in% names(marker_result)) {
    expr_df <- marker_result$marker_expr_by_group
  } else if (is.data.frame(marker_result)) {
    expr_df <- marker_result
  } else {
    stop(
      "marker_result must be either the output list from CellWindX_TopMarkersStats() ",
      "or a data.frame containing marker expression statistics."
    )
  }

  required_cols <- c(
    "source_cluster",
    "target_cluster",
    "gene",
    value_col
  )

  missing_cols <- setdiff(required_cols, colnames(expr_df))

  if (length(missing_cols) > 0) {
    stop(
      "The following required columns are missing: ",
      paste(missing_cols, collapse = ", ")
    )
  }

  expr_df <- as.data.frame(expr_df)
  expr_df$source_cluster <- as.character(expr_df$source_cluster)
  expr_df$target_cluster <- as.character(expr_df$target_cluster)
  expr_df$gene <- as.character(expr_df$gene)
  expr_df[[value_col]] <- as.numeric(expr_df[[value_col]])

  # -----------------------------
  # 4. Add marker rank if available
  # -----------------------------
  gene_rank_df <- NULL

  if (is.list(marker_result) && "top_markers" %in% names(marker_result)) {

    top_markers <- marker_result$top_markers

    if (all(c("cluster", "gene") %in% colnames(top_markers))) {

      top_markers$cluster <- as.character(top_markers$cluster)
      top_markers$gene <- as.character(top_markers$gene)

      if (!"rank_in_cluster" %in% colnames(top_markers)) {
        top_markers$rank_in_cluster <- ave(
          seq_len(nrow(top_markers)),
          top_markers$cluster,
          FUN = seq_along
        )
      }

      gene_rank_df <- top_markers[, c("cluster", "gene", "rank_in_cluster")]
      colnames(gene_rank_df) <- c("source_cluster", "gene", "rank_in_cluster")
    }
  }

  if (is.null(gene_rank_df)) {
    gene_rank_df <- unique(expr_df[, c("source_cluster", "gene")])
    gene_rank_df$rank_in_cluster <- ave(
      seq_len(nrow(gene_rank_df)),
      gene_rank_df$source_cluster,
      FUN = seq_along
    )
  }

  # -----------------------------
  # 5. Create unique marker column ID
  # -----------------------------
  expr_df$marker_id <- paste(expr_df$source_cluster, expr_df$gene, sep = "__")
  gene_rank_df$marker_id <- paste(gene_rank_df$source_cluster, gene_rank_df$gene, sep = "__")

  expr_df <- expr_df[
    !duplicated(expr_df[, c("target_cluster", "marker_id")]),
    ,
    drop = FALSE
  ]

  # -----------------------------
  # 6. Construct matrix
  # -----------------------------
  mat_df <- expr_df[, c("target_cluster", "marker_id", value_col)]

  mat_wide <- tidyr::pivot_wider(
    mat_df,
    names_from = "marker_id",
    values_from = value_col
  )

  mat_wide <- as.data.frame(mat_wide)
  rownames(mat_wide) <- mat_wide$target_cluster
  mat_wide$target_cluster <- NULL

  mat <- as.matrix(mat_wide)
  storage.mode(mat) <- "numeric"
  mat[is.na(mat)] <- 0

  # -----------------------------
  # 7. Order rows and columns
  # -----------------------------
  target_order <- unique(expr_df$target_cluster)
  target_order <- target_order[target_order %in% rownames(mat)]

  gene_rank_df <- gene_rank_df[
    gene_rank_df$marker_id %in% colnames(mat),
    ,
    drop = FALSE
  ]

  gene_rank_df <- gene_rank_df[
    order(gene_rank_df$source_cluster, gene_rank_df$rank_in_cluster),
    ,
    drop = FALSE
  ]

  marker_order <- unique(gene_rank_df$marker_id)
  marker_order <- marker_order[marker_order %in% colnames(mat)]

  mat <- mat[target_order, marker_order, drop = FALSE]

  gene_info <- gene_rank_df[
    match(colnames(mat), gene_rank_df$marker_id),
    ,
    drop = FALSE
  ]

  gene_labels <- gene_info$gene
  source_cluster_vec <- gene_info$source_cluster
  names(source_cluster_vec) <- colnames(mat)

  # -----------------------------
  # 8. Scaling
  # -----------------------------
  if (scale_method == "zscore") {

    mat_scaled <- scale(mat)
    mat_scaled[is.na(mat_scaled)] <- 0

    if (!is.null(zscore_clip)) {
      mat_scaled[mat_scaled > zscore_clip] <- zscore_clip
      mat_scaled[mat_scaled < -zscore_clip] <- -zscore_clip
    }

    plot_mat <- mat_scaled
    legend_name <- "Z-score"
    color_breaks <- c(-zscore_clip, 0, zscore_clip)

  } else {

    plot_mat <- mat

    if (value_col == "pct_expr") {
      legend_name <- "Percent"
    } else {
      legend_name <- value_col
    }

    color_breaks <- c(
      min(plot_mat, na.rm = TRUE),
      stats::median(plot_mat, na.rm = TRUE),
      max(plot_mat, na.rm = TRUE)
    )
  }

  if (length(unique(color_breaks)) < 3) {
    color_breaks <- c(
      min(plot_mat, na.rm = TRUE),
      mean(plot_mat, na.rm = TRUE),
      max(plot_mat, na.rm = TRUE)
    )
    color_breaks <- jitter(color_breaks, factor = 1e-8)
  }

  # -----------------------------
  # 9. Cluster annotation colors
  # -----------------------------
  source_levels <- unique(source_cluster_vec)

  if (length(source_levels) > length(pal_use$discrete)) {
    stop("The built-in palette supports up to 20 groups.")
  }

  source_colors <- pal_use$discrete[seq_along(source_levels)]
  names(source_colors) <- source_levels

  if (is.null(heatmap_title)) {
    heatmap_title <- paste0(
      "CellWindX marker heatmap | ",
      value_col,
      " | ",
      scale_method,
      " | ",
      palette
    )
  }

  # -----------------------------
  # 10. ComplexHeatmap engine
  # -----------------------------
  if (plot_engine == "complex") {

    col_fun <- circlize::colorRamp2(
      color_breaks,
      c(
        pal_use$heatmap_low,
        pal_use$heatmap_mid,
        pal_use$heatmap_high
      )
    )

    top_anno <- ComplexHeatmap::HeatmapAnnotation(
      MarkerSource = ComplexHeatmap::anno_simple(
        source_cluster_vec,
        col = source_colors,
        height = grid::unit(2.2, "mm")
      ),
      show_annotation_name = FALSE
    )

    if (round_cell) {

      ht <- ComplexHeatmap::Heatmap(
        plot_mat,
        name = legend_name,
        col = col_fun,
        cluster_rows = cluster_rows,
        cluster_columns = cluster_columns,
        show_row_names = show_row_names,
        show_column_names = show_column_names,
        column_labels = gene_labels,
        column_names_rot = column_names_rot,
        column_split = factor(source_cluster_vec, levels = source_levels),
        top_annotation = top_anno,
        rect_gp = grid::gpar(col = NA, fill = NA),
        cell_fun = function(j, i, x, y, w, h, fill) {
          v <- plot_mat[i, j]
          grid::grid.roundrect(
            x = x,
            y = y,
            width = w * 0.90,
            height = h * 0.90,
            r = grid::unit(0.35, "snpc"),
            gp = grid::gpar(
              fill = col_fun(v),
              col = NA
            )
          )
        },
        use_raster = FALSE,
        width = ncol(plot_mat) * grid::unit(cell_width_mm, "mm"),
        height = nrow(plot_mat) * grid::unit(cell_height_mm, "mm"),
        row_names_gp = grid::gpar(fontsize = row_fontsize),
        column_names_gp = grid::gpar(fontsize = column_fontsize),
        column_title_gp = grid::gpar(fontsize = 9, fontface = "bold"),
        heatmap_legend_param = list(
          title = legend_name,
          title_gp = grid::gpar(fontsize = 9, fontface = "bold"),
          labels_gp = grid::gpar(fontsize = 8)
        )
      )

    } else {

      ht <- ComplexHeatmap::Heatmap(
        plot_mat,
        name = legend_name,
        col = col_fun,
        cluster_rows = cluster_rows,
        cluster_columns = cluster_columns,
        show_row_names = show_row_names,
        show_column_names = show_column_names,
        column_labels = gene_labels,
        column_names_rot = column_names_rot,
        column_split = factor(source_cluster_vec, levels = source_levels),
        top_annotation = top_anno,
        rect_gp = grid::gpar(col = "white", lwd = 0.7),
        width = ncol(plot_mat) * grid::unit(cell_width_mm, "mm"),
        height = nrow(plot_mat) * grid::unit(cell_height_mm, "mm"),
        row_names_gp = grid::gpar(fontsize = row_fontsize),
        column_names_gp = grid::gpar(fontsize = column_fontsize),
        column_title_gp = grid::gpar(fontsize = 9, fontface = "bold"),
        heatmap_legend_param = list(
          title = legend_name,
          title_gp = grid::gpar(fontsize = 9, fontface = "bold"),
          labels_gp = grid::gpar(fontsize = 8)
        )
      )
    }

    if (draw) {
      ComplexHeatmap::draw(
        ht,
        heatmap_legend_side = legend_position,
        annotation_legend_side = legend_position
      )
    }

    if (!is.null(output_file)) {

      if (grepl("\\.pdf$", output_file, ignore.case = TRUE)) {
        grDevices::pdf(output_file, width = output_width, height = output_height)
        ComplexHeatmap::draw(
          ht,
          heatmap_legend_side = legend_position,
          annotation_legend_side = legend_position
        )
        grDevices::dev.off()
      } else if (grepl("\\.png$", output_file, ignore.case = TRUE)) {
        grDevices::png(
          output_file,
          width = output_width,
          height = output_height,
          units = "in",
          res = dpi
        )
        ComplexHeatmap::draw(
          ht,
          heatmap_legend_side = legend_position,
          annotation_legend_side = legend_position
        )
        grDevices::dev.off()
      } else {
        warning("Only .pdf and .png output_file are supported for plot_engine = 'complex'.")
      }
    }

    result <- list(
      plot = ht,
      matrix_raw = mat,
      matrix_plot = plot_mat,
      gene_info = gene_info,
      source_colors = source_colors,
      palette = palette,
      plot_engine = plot_engine,
      value_col = value_col,
      scale_method = scale_method
    )

    class(result) <- c("CellWindX_MarkerHeatmap", class(result))
    return(result)
  }

  # -----------------------------
  # 11. ggplot2 engine
  # -----------------------------
  if (plot_engine == "ggplot") {

    plot_df <- as.data.frame(plot_mat)
    plot_df$target_cluster <- rownames(plot_mat)

    plot_df <- tidyr::pivot_longer(
      plot_df,
      cols = -target_cluster,
      names_to = "marker_id",
      values_to = "value"
    )

    gene_info2 <- gene_info[, c("marker_id", "source_cluster", "gene")]

    plot_df <- dplyr::left_join(
      plot_df,
      gene_info2,
      by = "marker_id"
    )

    plot_df$target_cluster <- factor(
      plot_df$target_cluster,
      levels = rev(rownames(plot_mat))
    )

    plot_df$marker_id <- factor(
      plot_df$marker_id,
      levels = colnames(plot_mat)
    )

    plot_df$gene <- factor(
      plot_df$gene,
      levels = unique(gene_info$gene)
    )

    plot_df$source_cluster <- factor(
      plot_df$source_cluster,
      levels = source_levels
    )

    p <- ggplot2::ggplot(
      plot_df,
      ggplot2::aes(
        x = marker_id,
        y = target_cluster,
        fill = value
      )
    ) +
      ggplot2::geom_tile(
        color = "white",
        linewidth = 0.35
      ) +
      ggplot2::scale_x_discrete(
        labels = stats::setNames(gene_info$gene, gene_info$marker_id)
      ) +
      ggplot2::scale_fill_gradient2(
        low = pal_use$heatmap_low,
        mid = pal_use$heatmap_mid,
        high = pal_use$heatmap_high,
        midpoint = 0,
        name = legend_name
      ) +
      ggplot2::facet_grid(
        . ~ source_cluster,
        scales = "free_x",
        space = "free_x"
      ) +
      ggplot2::labs(
        x = NULL,
        y = NULL,
        title = heatmap_title
      ) +
      ggplot2::theme_minimal(base_size = 12) +
      ggplot2::theme(
        plot.title = ggplot2::element_text(
          hjust = 0.5,
          face = "bold",
          size = 14
        ),
        panel.grid = ggplot2::element_blank(),
        axis.text.x = ggplot2::element_text(
          size = column_fontsize,
          angle = column_names_rot,
          hjust = 1,
          vjust = 1
        ),
        axis.text.y = ggplot2::element_text(
          size = row_fontsize,
          color = "black"
        ),
        strip.text.x = ggplot2::element_text(
          size = 10,
          face = "bold"
        ),
        panel.spacing.x = grid::unit(0.15, "cm"),
        legend.position = ifelse(
          legend_position %in% c("right", "left", "top", "bottom", "none"),
          legend_position,
          "right"
        ),
        legend.title = ggplot2::element_text(
          size = 9,
          face = "bold"
        ),
        legend.text = ggplot2::element_text(size = 8),
        plot.margin = ggplot2::margin(8, 8, 8, 8)
      )

    if (!is.null(output_file)) {
      ggplot2::ggsave(
        filename = output_file,
        plot = p,
        width = output_width,
        height = output_height,
        dpi = dpi
      )
    }

    result <- list(
      plot = p,
      matrix_raw = mat,
      matrix_plot = plot_mat,
      plot_data = plot_df,
      gene_info = gene_info,
      source_colors = source_colors,
      palette = palette,
      plot_engine = plot_engine,
      value_col = value_col,
      scale_method = scale_method
    )

    class(result) <- c("CellWindX_MarkerHeatmap", class(result))
    return(result)
  }
}

#' Draw gene-level radar plots across annotated cell groups
#'
#' @description
#' `CellWindX_GeneRadar()` visualizes the expression pattern of selected genes
#' across annotated cell groups using straight-line spider radar plots. For each
#' selected gene, the function generates two radar plots: one for the percentage
#' of expressing cells and another for average expression.
#'
#' @details
#' This function is designed to directly accept the output generated by
#' `CellWindX_TopMarkersStats()`. It uses the `marker_expr_by_group` table from
#' that result object, which should contain gene expression statistics across
#' cell groups.
#'
#' Each cell group is represented as one axis of the radar plot. Unlike polar
#' coordinate radar plots, this function manually calculates polygon coordinates,
#' resulting in straight polygon grid lines rather than curved circular grid
#' lines. This style is generally clearer for comparing cell-group-specific
#' expression patterns.
#'
#' The function draws two panels:
#'
#' \itemize{
#'   \item `pct_plot`: expression percentage across cell groups.
#'   \item `avg_plot`: average expression across cell groups.
#' }
#'
#' The expression percentage and average expression panels use intentionally
#' distinct color sets within the selected CellWindX palette. Three built-in
#' palettes are available: `"shanshui"`, `"chongqing_modern"`, and `"girlish"`.
#'
#' @param marker_result A result object returned by `CellWindX_TopMarkersStats()`,
#'   or a data frame containing marker expression statistics. If a data frame is
#'   provided, it must contain the columns `source_cluster`, `target_cluster`,
#'   `gene`, `pct_expr`, and `avg_expr`.
#' @param genes Character vector. Gene symbols to visualize.
#' @param source_cluster Character vector or `NULL`. Optional source-cluster
#'   filter. If provided, only marker statistics from the selected source
#'   cluster(s) are used. Default is `NULL`.
#' @param aggregate_fun Character string. Method used to aggregate duplicated
#'   gene-cell-group rows. One of `"mean"`, `"max"`, or `"first"`.
#'   Default is `"mean"`.
#' @param palette Character string. Built-in CellWindX palette. One of
#'   `"shanshui"`, `"chongqing_modern"`, or `"girlish"`.
#' @param scale_avg Logical. Whether to scale average expression values before
#'   plotting. Default is `TRUE`.
#' @param scale_pct Logical. Whether to scale expression percentage values by
#'   gene before plotting. If `FALSE`, the raw percentage values are used and
#'   capped by `pct_max`. Default is `FALSE`.
#' @param pct_max Numeric. Maximum value used to cap percentage values when
#'   `scale_pct = FALSE`. Default is `100`.
#' @param avg_scale_method Character string. Scaling method for average
#'   expression. One of `"max"`, `"zscore"`, or `"none"`. `"max"` rescales
#'   each gene to a 0-100 range by dividing by its maximum value. `"zscore"`
#'   performs gene-wise Z-score scaling and rescales the result to 0-100.
#'   `"none"` keeps the original average expression values. Default is `"max"`.
#' @param grid_levels Numeric vector. Radar grid levels. These values determine
#'   the radius of the straight polygon grid lines. Default is
#'   `c(25, 50, 75, 100)`.
#' @param axis_label_size Numeric. Font size of cell-group labels placed around
#'   the radar plot. Default is `5.2`.
#' @param grid_label_size Numeric. Font size of radar grid labels. Default is
#'   `3.2`.
#' @param facet_title_size Numeric. Font size of facet titles when multiple
#'   genes are plotted. Default is `12`.
#' @param line_width Numeric. Width of radar polygon lines. Default is `1.2`.
#' @param point_size Numeric. Size of points on radar axes. Default is `2.8`.
#' @param fill_alpha Numeric. Transparency of polygon fill. Values should
#'   usually range from 0 to 1. Default is `0.18`.
#' @param facet Logical. Whether to draw multiple genes as separate facets.
#'   Default is `TRUE`.
#' @param ncol Integer or `NULL`. Number of columns used in facet layout when
#'   `facet = TRUE`. Default is `NULL`.
#' @param show_points Logical. Whether to show points on radar axes.
#'   Default is `TRUE`.
#' @param show_fill Logical. Whether to fill radar polygons. Default is `TRUE`.
#' @param show_grid_label Logical. Whether to show numeric grid labels.
#'   Default is `TRUE`.
#' @param output_file Character string or `NULL`. Optional path used to save the
#'   combined radar plot. Supported formats depend on [ggplot2::ggsave()],
#'   such as `.pdf`, `.png`, `.tiff`, or `.svg`. Default is `NULL`.
#' @param output_width Numeric. Width of the saved plot in inches. Default is
#'   `11`.
#' @param output_height Numeric. Height of the saved plot in inches. Default is
#'   `5.5`.
#' @param dpi Numeric. Resolution used when saving raster formats. Default is
#'   `300`.
#'
#' @return A list with class `"CellWindX_GeneRadar"` containing:
#' \describe{
#'   \item{pct_plot}{A ggplot object showing expression percentage.}
#'   \item{avg_plot}{A ggplot object showing average expression.}
#'   \item{combined_plot}{A patchwork object combining percentage and average expression plots.}
#'   \item{plot_data}{A data frame used for plotting.}
#'   \item{pct_coord}{Radar coordinates for the expression percentage plot.}
#'   \item{avg_coord}{Radar coordinates for the average expression plot.}
#'   \item{palette}{The selected CellWindX palette.}
#'   \item{genes}{Genes used for visualization.}
#'   \item{parameters}{A list of function parameters used in the plot.}
#' }
#'
#' @author Xiaofeng Yang, Chongqing Medical University
#'
#' @examples
#' marker_df <- data.frame(
#'   source_cluster = rep(c("T cell", "B cell"), each = 4),
#'   target_cluster = rep(c("T cell", "B cell", "Myeloid", "Platelet"), times = 2),
#'   gene = rep(c("CD3D", "MS4A1"), each = 4),
#'   pct_expr = c(92, 8, 15, 4, 6, 88, 12, 3),
#'   avg_expr = c(2.8, 0.2, 0.5, 0.1, 0.1, 2.5, 0.4, 0.1),
#'   stringsAsFactors = FALSE
#' )
#'
#' radar_res <- CellWindX_GeneRadar(
#'   marker_result = marker_df,
#'   genes = c("CD3D", "MS4A1"),
#'   palette = "shanshui",
#'   scale_avg = TRUE,
#'   avg_scale_method = "max",
#'   facet = TRUE,
#'   ncol = 2
#' )
#'
#' radar_res$combined_plot
#'
#' @export
CellWindX_GeneRadar <- function(
    marker_result,
    genes,
    source_cluster = NULL,
    aggregate_fun = c("mean", "max", "first"),
    palette = c("shanshui", "chongqing_modern", "girlish"),
    scale_avg = TRUE,
    scale_pct = FALSE,
    pct_max = 100,
    avg_scale_method = c("max", "zscore", "none"),
    grid_levels = c(25, 50, 75, 100),
    axis_label_size = 5.2,
    grid_label_size = 3.2,
    facet_title_size = 12,
    line_width = 1.2,
    point_size = 2.8,
    fill_alpha = 0.18,
    facet = TRUE,
    ncol = NULL,
    show_points = TRUE,
    show_fill = TRUE,
    show_grid_label = TRUE,
    output_file = NULL,
    output_width = 11,
    output_height = 5.5,
    dpi = 300
) {

  # -----------------------------
  # 1. Package check
  # -----------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.")
  }
  if (!requireNamespace("dplyr", quietly = TRUE)) {
    stop("Package 'dplyr' is required.")
  }
  if (!requireNamespace("tidyr", quietly = TRUE)) {
    stop("Package 'tidyr' is required.")
  }
  if (!requireNamespace("patchwork", quietly = TRUE)) {
    stop("Package 'patchwork' is required. Please install.packages('patchwork').")
  }

  aggregate_fun <- match.arg(aggregate_fun)
  palette <- match.arg(palette)
  avg_scale_method <- match.arg(avg_scale_method)

  if (missing(genes) || length(genes) == 0) {
    stop("'genes' must be provided.")
  }

  genes <- unique(as.character(genes))

  # -----------------------------
  # 2. CellWindX palettes
  # pct_colors and avg_colors are intentionally different
  # -----------------------------
  CellWindX_palettes <- list(

    shanshui = list(
      pct_colors = c(
        "#6F8F9F", "#8FB9A8", "#A7C7D9", "#7FA6C7", "#738A99",
        "#AFC8BA", "#C9D6DF", "#8A9A8F", "#6E7F80", "#B7C8A3",
        "#A6B1C2", "#C6D8D3", "#7C9A92", "#9BB8C2", "#5F7F8A",
        "#B5C7B1", "#91B3A2", "#82A6B3", "#D0DDD7", "#789395"
      ),
      avg_colors = c(
        "#9D9AC7", "#B58C9E", "#C6B7D9", "#D1A7A0", "#9E8FB2",
        "#BFA58A", "#D8C7A3", "#E2D8C3", "#B5A789", "#C7A6B8",
        "#8E89B8", "#BA99AD", "#D6C6B9", "#A88FAF", "#C9B6D8",
        "#B59B8F", "#D9B7B3", "#A696C8", "#CBBDA4", "#907EA4"
      )
    ),

    chongqing_modern = list(
      pct_colors = c(
        "#2F4858", "#33658A", "#4A90A4", "#86BBD8", "#1C7293",
        "#00A6A6", "#3A506B", "#355070", "#7D8491", "#B8B8D1",
        "#264653", "#287271", "#457B9D", "#118AB2", "#073B4C",
        "#52796F", "#354F52", "#669BBC", "#003049", "#5C677D"
      ),
      avg_colors = c(
        "#F26419", "#F6AE2D", "#D64550", "#8E5572", "#5D5179",
        "#E0A458", "#C8553D", "#6D597A", "#758E4F", "#2A2D34",
        "#E76F51", "#F4A261", "#E63946", "#9D4EDD", "#FF7F51",
        "#BC4749", "#B56576", "#C77DFF", "#FFB703", "#A44A3F"
      )
    ),

    girlish = list(
      pct_colors = c(
        "#A0C4FF", "#BFD7FF", "#A0E7E5", "#B5EAD7", "#B4F8C8",
        "#D4F0F0", "#CCE2CB", "#C7CEEA", "#CDE7FF", "#BDE0FE",
        "#90DBF4", "#98F5E1", "#8EECF5", "#C1FBA4", "#B8E0D2",
        "#AED9E0", "#CDEAC0", "#A2D2FF", "#CAF0F8", "#B9FBC0"
      ),
      avg_colors = c(
        "#FF8FAB", "#FF9AA2", "#FFB7B2", "#FFD6E0", "#F6C1D5",
        "#F7A8B8", "#D5AAFF", "#F3C4FB", "#E7C6FF", "#FFABAB",
        "#FFC3A0", "#FFDAC1", "#FBE7C6", "#FFAFCC", "#FFC8DD",
        "#FFCAD4", "#F4ACB7", "#E0BBE4", "#FFB5A7", "#FFCCD5"
      )
    )
  )

  pal_use <- CellWindX_palettes[[palette]]

  # -----------------------------
  # 3. Extract marker expression table
  # -----------------------------
  if (is.list(marker_result) && "marker_expr_by_group" %in% names(marker_result)) {
    expr_df <- marker_result$marker_expr_by_group
  } else if (is.data.frame(marker_result)) {
    expr_df <- marker_result
  } else {
    stop(
      "marker_result must be either the output list from CellWindX_TopMarkersStats() ",
      "or a data.frame containing marker expression statistics."
    )
  }

  required_cols <- c(
    "source_cluster",
    "target_cluster",
    "gene",
    "pct_expr",
    "avg_expr"
  )

  missing_cols <- setdiff(required_cols, colnames(expr_df))

  if (length(missing_cols) > 0) {
    stop(
      "The following required columns are missing: ",
      paste(missing_cols, collapse = ", ")
    )
  }

  expr_df <- as.data.frame(expr_df)
  expr_df$source_cluster <- as.character(expr_df$source_cluster)
  expr_df$target_cluster <- as.character(expr_df$target_cluster)
  expr_df$gene <- as.character(expr_df$gene)
  expr_df$pct_expr <- as.numeric(expr_df$pct_expr)
  expr_df$avg_expr <- as.numeric(expr_df$avg_expr)

  if (!is.null(source_cluster)) {
    expr_df <- expr_df[expr_df$source_cluster %in% source_cluster, , drop = FALSE]
  }

  expr_df <- expr_df[expr_df$gene %in% genes, , drop = FALSE]

  if (nrow(expr_df) == 0) {
    stop(
      "None of the selected genes were found in marker_result. ",
      "Please check genes or source_cluster."
    )
  }

  missing_genes <- setdiff(genes, unique(expr_df$gene))

  if (length(missing_genes) > 0) {
    warning(
      "The following genes were not found and will be ignored: ",
      paste(missing_genes, collapse = ", ")
    )
  }

  # -----------------------------
  # 4. Aggregate duplicated gene-target rows
  # -----------------------------
  if (aggregate_fun == "mean") {
    plot_df <- expr_df |>
      dplyr::group_by(gene, target_cluster) |>
      dplyr::summarise(
        pct_expr = mean(pct_expr, na.rm = TRUE),
        avg_expr = mean(avg_expr, na.rm = TRUE),
        .groups = "drop"
      )
  }

  if (aggregate_fun == "max") {
    plot_df <- expr_df |>
      dplyr::group_by(gene, target_cluster) |>
      dplyr::summarise(
        pct_expr = max(pct_expr, na.rm = TRUE),
        avg_expr = max(avg_expr, na.rm = TRUE),
        .groups = "drop"
      )
  }

  if (aggregate_fun == "first") {
    plot_df <- expr_df |>
      dplyr::group_by(gene, target_cluster) |>
      dplyr::slice(1) |>
      dplyr::ungroup() |>
      dplyr::select(gene, target_cluster, pct_expr, avg_expr)
  }

  # -----------------------------
  # 5. Preserve cell group order
  # -----------------------------
  cell_order <- unique(expr_df$target_cluster)
  cell_order <- cell_order[cell_order %in% unique(plot_df$target_cluster)]

  plot_df$target_cluster <- factor(
    plot_df$target_cluster,
    levels = cell_order
  )

  plot_df$gene <- factor(
    plot_df$gene,
    levels = genes[genes %in% unique(plot_df$gene)]
  )

  plot_df <- tidyr::complete(
    plot_df,
    gene,
    target_cluster,
    fill = list(
      pct_expr = 0,
      avg_expr = 0
    )
  )

  plot_df$target_cluster <- factor(
    plot_df$target_cluster,
    levels = cell_order
  )

  plot_df$gene <- factor(
    plot_df$gene,
    levels = genes[genes %in% unique(as.character(plot_df$gene))]
  )

  # -----------------------------
  # 6. Scaling
  # -----------------------------
  plot_df$pct_value <- plot_df$pct_expr

  if (scale_pct) {
    plot_df <- plot_df |>
      dplyr::group_by(gene) |>
      dplyr::mutate(
        pct_value = ifelse(
          max(pct_expr, na.rm = TRUE) == 0,
          0,
          pct_expr / max(pct_expr, na.rm = TRUE) * 100
        )
      ) |>
      dplyr::ungroup()
  } else {
    plot_df$pct_value <- pmin(plot_df$pct_expr, pct_max)
  }

  plot_df$avg_value <- plot_df$avg_expr

  if (scale_avg) {

    if (avg_scale_method == "max") {
      plot_df <- plot_df |>
        dplyr::group_by(gene) |>
        dplyr::mutate(
          avg_value = ifelse(
            max(avg_expr, na.rm = TRUE) == 0,
            0,
            avg_expr / max(avg_expr, na.rm = TRUE) * 100
          )
        ) |>
        dplyr::ungroup()
    }

    if (avg_scale_method == "zscore") {
      plot_df <- plot_df |>
        dplyr::group_by(gene) |>
        dplyr::mutate(
          avg_value = as.numeric(scale(avg_expr))
        ) |>
        dplyr::ungroup()

      plot_df$avg_value[is.na(plot_df$avg_value)] <- 0

      min_v <- min(plot_df$avg_value, na.rm = TRUE)
      max_v <- max(plot_df$avg_value, na.rm = TRUE)

      if (max_v == min_v) {
        plot_df$avg_value <- 0
      } else {
        plot_df$avg_value <- (plot_df$avg_value - min_v) / (max_v - min_v) * 100
      }
    }
  }

  if (!scale_avg || avg_scale_method == "none") {
    plot_df$avg_value <- plot_df$avg_expr
  }

  # -----------------------------
  # 7. Convert values to radar coordinates
  # -----------------------------
  add_radar_coordinates <- function(df, value_col) {

    n_axis <- length(cell_order)

    angle_df <- data.frame(
      target_cluster = factor(cell_order, levels = cell_order),
      axis_id = seq_along(cell_order),
      angle = seq(0, 2 * pi, length.out = n_axis + 1)[-1],
      stringsAsFactors = FALSE
    )

    df2 <- dplyr::left_join(
      df,
      angle_df,
      by = "target_cluster"
    )

    df2$radius <- as.numeric(df2[[value_col]])
    df2$x <- df2$radius * sin(df2$angle)
    df2$y <- df2$radius * cos(df2$angle)

    df2 <- df2 |>
      dplyr::arrange(gene, axis_id)

    # close polygon by repeating the first point of each gene
    df_closed <- df2 |>
      dplyr::group_by(gene) |>
      dplyr::group_modify(function(.x, .y) {
        rbind(.x, .x[1, , drop = FALSE])
      }) |>
      dplyr::ungroup()

    return(list(
      data = df2,
      data_closed = df_closed,
      angle_df = angle_df
    ))
  }

  pct_coord <- add_radar_coordinates(plot_df, "pct_value")
  avg_coord <- add_radar_coordinates(plot_df, "avg_value")

  # -----------------------------
  # 8. Grid data: straight polygon grid lines
  # -----------------------------
  build_grid_data <- function(angle_df, grid_levels) {

    grid_poly <- do.call(
      rbind,
      lapply(grid_levels, function(r) {
        tmp <- angle_df
        tmp$radius <- r
        tmp$x <- r * sin(tmp$angle)
        tmp$y <- r * cos(tmp$angle)
        tmp$grid_level <- r
        tmp <- tmp[order(tmp$axis_id), , drop = FALSE]
        rbind(tmp, tmp[1, , drop = FALSE])
      })
    )

    axis_df <- angle_df
    axis_df$x <- 0
    axis_df$y <- 0
    axis_df$xend <- max(grid_levels) * sin(axis_df$angle)
    axis_df$yend <- max(grid_levels) * cos(axis_df$angle)

    label_df <- angle_df
    label_radius <- max(grid_levels) * 1.18
    label_df$x <- label_radius * sin(label_df$angle)
    label_df$y <- label_radius * cos(label_df$angle)
    label_df$label <- as.character(label_df$target_cluster)

    grid_label_df <- data.frame(
      x = 0,
      y = grid_levels,
      label = grid_levels,
      stringsAsFactors = FALSE
    )

    return(list(
      grid_poly = grid_poly,
      axis_df = axis_df,
      label_df = label_df,
      grid_label_df = grid_label_df
    ))
  }

  radar_grid <- build_grid_data(
    angle_df = pct_coord$angle_df,
    grid_levels = grid_levels
  )

  # -----------------------------
  # 9. Plot helper
  # -----------------------------
  gene_levels <- levels(plot_df$gene)
  n_genes <- length(gene_levels)

  pct_gene_cols <- pal_use$pct_colors[seq_len(min(n_genes, length(pal_use$pct_colors)))]
  avg_gene_cols <- pal_use$avg_colors[seq_len(min(n_genes, length(pal_use$avg_colors)))]

  names(pct_gene_cols) <- gene_levels[seq_along(pct_gene_cols)]
  names(avg_gene_cols) <- gene_levels[seq_along(avg_gene_cols)]

  make_spider_plot <- function(
    coord_obj,
    value_type = c("pct", "avg"),
    title,
    gene_cols
  ) {

    value_type <- match.arg(value_type)

    df_point <- coord_obj$data
    df_closed <- coord_obj$data_closed

    p <- ggplot2::ggplot()

    # straight polygon grid
    p <- p +
      ggplot2::geom_path(
        data = radar_grid$grid_poly,
        ggplot2::aes(
          x = x,
          y = y,
          group = grid_level
        ),
        color = "grey82",
        linewidth = 0.35
      ) +
      ggplot2::geom_segment(
        data = radar_grid$axis_df,
        ggplot2::aes(
          x = x,
          y = y,
          xend = xend,
          yend = yend
        ),
        color = "grey86",
        linewidth = 0.35
      )

    # radar polygon
    if (show_fill) {
      p <- p +
        ggplot2::geom_polygon(
          data = df_closed,
          ggplot2::aes(
            x = x,
            y = y,
            group = gene,
            fill = gene
          ),
          alpha = fill_alpha,
          color = NA
        )
    }

    p <- p +
      ggplot2::geom_path(
        data = df_closed,
        ggplot2::aes(
          x = x,
          y = y,
          group = gene,
          color = gene
        ),
        linewidth = line_width,
        lineend = "round"
      )

    if (show_points) {
      p <- p +
        ggplot2::geom_point(
          data = df_point,
          ggplot2::aes(
            x = x,
            y = y,
            color = gene
          ),
          size = point_size,
          alpha = 0.95
        )
    }

    # axis labels: larger cell group labels
    p <- p +
      ggplot2::geom_text(
        data = radar_grid$label_df,
        ggplot2::aes(
          x = x,
          y = y,
          label = label
        ),
        size = axis_label_size,
        fontface = "bold",
        color = "black",
        lineheight = 0.9
      )

    if (show_grid_label) {
      p <- p +
        ggplot2::geom_text(
          data = radar_grid$grid_label_df,
          ggplot2::aes(
            x = x,
            y = y,
            label = label
          ),
          size = grid_label_size,
          color = "grey45",
          hjust = -0.2
        )
    }

    p <- p +
      ggplot2::scale_color_manual(values = gene_cols) +
      ggplot2::scale_fill_manual(values = gene_cols) +
      ggplot2::coord_equal(
        xlim = c(-max(grid_levels) * 1.35, max(grid_levels) * 1.35),
        ylim = c(-max(grid_levels) * 1.35, max(grid_levels) * 1.35),
        clip = "off"
      ) +
      ggplot2::labs(
        title = title,
        color = "Gene",
        fill = "Gene"
      ) +
      ggplot2::theme_void(base_size = 12) +
      ggplot2::theme(
        plot.title = ggplot2::element_text(
          hjust = 0.5,
          face = "bold",
          size = 15
        ),
        legend.position = "right",
        legend.title = ggplot2::element_text(
          face = "bold",
          size = 10
        ),
        legend.text = ggplot2::element_text(size = 9),
        plot.margin = ggplot2::margin(18, 18, 18, 18)
      )

    if (facet && n_genes > 1) {
      p <- p +
        ggplot2::facet_wrap(
          ~ gene,
          ncol = ncol
        ) +
        ggplot2::theme(
          legend.position = "none",
          strip.text = ggplot2::element_text(
            face = "bold",
            size = facet_title_size
          )
        )
    }

    return(p)
  }

  # -----------------------------
  # 10. Build plots
  # -----------------------------
  pct_title <- ifelse(
    scale_pct,
    "Scaled expression percentage",
    "Expression percentage"
  )

  avg_title <- ifelse(
    scale_avg,
    paste0("Scaled average expression", " (", avg_scale_method, ")"),
    "Average expression"
  )

  pct_plot <- make_spider_plot(
    coord_obj = pct_coord,
    value_type = "pct",
    title = pct_title,
    gene_cols = pct_gene_cols
  )

  avg_plot <- make_spider_plot(
    coord_obj = avg_coord,
    value_type = "avg",
    title = avg_title,
    gene_cols = avg_gene_cols
  )

  combined_plot <- pct_plot + avg_plot + patchwork::plot_layout(ncol = 2)

  # -----------------------------
  # 11. Save output
  # -----------------------------
  if (!is.null(output_file)) {
    ggplot2::ggsave(
      filename = output_file,
      plot = combined_plot,
      width = output_width,
      height = output_height,
      dpi = dpi
    )
  }

  # -----------------------------
  # 12. Return
  # -----------------------------
  result <- list(
    pct_plot = pct_plot,
    avg_plot = avg_plot,
    combined_plot = combined_plot,
    plot_data = plot_df,
    pct_coord = pct_coord$data,
    avg_coord = avg_coord$data,
    palette = palette,
    genes = genes,
    parameters = list(
      source_cluster = source_cluster,
      aggregate_fun = aggregate_fun,
      scale_avg = scale_avg,
      scale_pct = scale_pct,
      avg_scale_method = avg_scale_method,
      pct_max = pct_max,
      grid_levels = grid_levels
    )
  )

  class(result) <- c("CellWindX_GeneRadar", class(result))

  return(result)
}

#' Visualize Seurat embeddings with CellWindX palettes
#'
#' @description
#' `CellWindX_DimPlot()` visualizes UMAP or t-SNE embeddings from a processed
#' Seurat object using built-in CellWindX color palettes. The function is designed
#' for annotated single-cell objects and supports three predefined aesthetic
#' styles: Chinese landscape-inspired, Chongqing modern, and girlish palettes.
#'
#' @details
#' This function is a wrapper around [Seurat::DimPlot()] with customized color
#' palettes and publication-oriented theme settings. It requires that the
#' selected dimensional reduction has already been computed and stored in
#' `object@reductions`, such as by running [Seurat::RunUMAP()] or
#' [Seurat::RunTSNE()].
#'
#' The built-in palettes support up to 20 annotated groups. If the number of
#' groups in `group.by` exceeds 20, the function will return an error.
#'
#' @param object A Seurat object containing dimensional reduction results.
#' @param group.by Character string. Metadata column used to color cells.
#'   Default is `"seurat_annotations"`.
#' @param reduction Character string. Dimensional reduction to visualize.
#'   One of `"umap"` or `"tsne"`.
#' @param palette Character string. Built-in CellWindX palette to use.
#'   One of `"shanshui"`, `"chongqing_modern"`, or `"girlish"`.
#' @param label Logical. Whether to show group labels on the plot.
#'   Default is `TRUE`.
#' @param repel Logical. Whether to repel text labels using ggrepel.
#'   Default is `TRUE`.
#' @param pt.size Numeric. Point size passed to [Seurat::DimPlot()].
#'   Default is `0.6`.
#' @param label.size Numeric. Label font size. Reserved for future extension.
#'   Default is `4`.
#' @param alpha Numeric or `NULL`. Point transparency. Values should usually
#'   range from 0 to 1. Default is `0.9`.
#' @param shuffle Logical. Whether to randomly shuffle plotting order of cells.
#'   Default is `TRUE`.
#' @param seed Integer. Random seed used when `shuffle = TRUE`.
#'   Default is `123`.
#' @param title Character string or `NULL`. Plot title. If `NULL`, a default
#'   title is automatically generated.
#' @param legend.position Character string. Legend position passed to
#'   [ggplot2::theme()]. Common values include `"right"`, `"left"`,
#'   `"bottom"`, `"top"`, and `"none"`. Default is `"right"`.
#'
#' @return A ggplot object generated from [Seurat::DimPlot()].
#'
#' @author Xiaofeng Yang, Chongqing Medical University
#'
#' @examples
#' data("pbmc_small", package = "SeuratObject")
#'
#' p1 <- CellWindX_DimPlot(
#'   object = pbmc_small,
#'   group.by = "groups",
#'   reduction = "tsne",
#'   palette = "shanshui"
#' )
#'
#' p1
#'
#' @export
CellWindX_DimPlot <- function(
    object,
    group.by = "seurat_annotations",
    reduction = c("umap", "tsne"),
    palette = c("shanshui", "chongqing_modern", "girlish"),
    label = TRUE,
    repel = TRUE,
    pt.size = 0.6,
    label.size = 4,
    alpha = 0.9,
    shuffle = TRUE,
    seed = 123,
    title = NULL,
    legend.position = "right"
) {

  # -----------------------------
  # 1. Check packages
  # -----------------------------
  if (!requireNamespace("Seurat", quietly = TRUE)) {
    stop("Package 'Seurat' is required.")
  }
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.")
  }

  # -----------------------------
  # 2. Match arguments
  # -----------------------------
  reduction <- match.arg(reduction)
  palette <- match.arg(palette)

  # -----------------------------
  # 3. Check Seurat object
  # -----------------------------
  if (!inherits(object, "Seurat")) {
    stop("'object' must be a Seurat object.")
  }

  if (!group.by %in% colnames(object@meta.data)) {
    stop(
      paste0(
        "The metadata column '", group.by, "' was not found in object@meta.data."
      )
    )
  }

  if (!reduction %in% names(object@reductions)) {
    stop(
      paste0(
        "Reduction '", reduction, "' was not found in object@reductions. ",
        "Please run RunUMAP() or RunTSNE() first."
      )
    )
  }

  # -----------------------------
  # 4. Built-in CellWindX palettes
  # -----------------------------
  CellWindX_palettes <- list(

    # 中国山水画：淡雅、青绿、蓝紫、墨灰、胭脂点缀
    shanshui = c(
      "#8FB9A8", "#B7C8A3", "#A7C7D9", "#7FA6C7", "#9D9AC7",
      "#C6B7D9", "#D8C7A3", "#BFA58A", "#8A9A8F", "#6F8F9F",
      "#A6B1C2", "#C9D6DF", "#D6C6B9", "#E2D8C3", "#9E8FB2",
      "#738A99", "#B5A789", "#AFC8BA", "#D1A7A0", "#6E7F80"
    ),

    # 重庆摩登风格：夜景、霓虹、江水蓝、桥梁橙、都市灰
    chongqing_modern = c(
      "#2F4858", "#33658A", "#4A90A4", "#86BBD8", "#758E4F",
      "#F6AE2D", "#F26419", "#D64550", "#8E5572", "#5D5179",
      "#3A506B", "#1C7293", "#00A6A6", "#7D8491", "#B8B8D1",
      "#E0A458", "#C8553D", "#6D597A", "#355070", "#2A2D34"
    ),

    # 少女心：年轻、明亮、柔和、活力
    girlish = c(
      "#FF9AA2", "#FFB7B2", "#FFDAC1", "#E2F0CB", "#B5EAD7",
      "#C7CEEA", "#F6C1D5", "#D5AAFF", "#A0E7E5", "#B4F8C8",
      "#FBE7C6", "#F7A8B8", "#F3C4FB", "#BFD7FF", "#FFD6E0",
      "#FFABAB", "#FFC3A0", "#D4F0F0", "#CCE2CB", "#E7C6FF"
    )
  )

  colors_use <- CellWindX_palettes[[palette]]

  # -----------------------------
  # 5. Prepare group information
  # -----------------------------
  group_vec <- object@meta.data[[group.by]]
  group_vec <- as.factor(group_vec)
  group_levels <- levels(group_vec)
  n_groups <- length(group_levels)

  if (n_groups > length(colors_use)) {
    stop(
      paste0(
        "The selected palette supports up to ", length(colors_use),
        " groups, but '", group.by, "' contains ", n_groups, " groups. ",
        "Please provide fewer groups or extend the palette."
      )
    )
  }

  colors_use <- colors_use[seq_len(n_groups)]
  names(colors_use) <- group_levels

  # -----------------------------
  # 6. Default title
  # -----------------------------
  if (is.null(title)) {
    title <- paste0(
      toupper(reduction),
      " colored by ",
      group.by,
      " | palette: ",
      palette
    )
  }

  # -----------------------------
  # 7. Plot
  # -----------------------------
  p <- Seurat::DimPlot(
    object = object,
    reduction = reduction,
    group.by = group.by,
    cols = colors_use,
    label = label,
    repel = repel,
    pt.size = pt.size,
    label.size = label.size,
    shuffle = shuffle,
    seed = seed
  ) +
    ggplot2::ggtitle(title) +
    ggplot2::theme_classic(base_size = 14) +
    ggplot2::theme(
      plot.title = ggplot2::element_text(
        hjust = 0.5,
        face = "bold",
        size = 15
      ),
      axis.title = ggplot2::element_text(
        face = "bold",
        size = 13
      ),
      axis.text = ggplot2::element_text(
        color = "black",
        size = 11
      ),
      legend.title = ggplot2::element_blank(),
      legend.text = ggplot2::element_text(size = 11),
      legend.position = legend.position,
      panel.border = ggplot2::element_rect(
        color = "black",
        fill = NA,
        linewidth = 0.6
      ),
      axis.line = ggplot2::element_line(linewidth = 0.4),
      plot.margin = ggplot2::margin(10, 10, 10, 10)
    )

  # Seurat DimPlot 不直接支持 alpha，这里通过修改图层实现
  if (!is.null(alpha)) {
    for (i in seq_along(p$layers)) {
      if ("GeomPoint" %in% class(p$layers[[i]]$geom)) {
        p$layers[[i]]$aes_params$alpha <- alpha
      }
    }
  }

  return(p)
}

#' Identify top marker genes and summarize expression statistics
#'
#' @description
#' `CellWindX_TopMarkersStats()` identifies the top marker genes for each
#' annotated cell group in a Seurat object and summarizes their expression
#' percentage and average expression. The output is designed to be directly used
#' by downstream CellWindX visualization functions, including
#' `CellWindX_MarkerHeatmap()` and `CellWindX_GeneRadar()`.
#'
#' @details
#' This function first runs [Seurat::FindAllMarkers()] using the cell-group
#' annotation specified by `group.by`. For each group, the top `N` marker genes
#' are selected according to adjusted P value, log-fold change, and detection
#' percentage.
#'
#' For each selected marker gene, the function then calculates:
#'
#' \itemize{
#'   \item `n_cells`: number of cells in the target group.
#'   \item `n_expr_cells`: number of cells with expression greater than
#'   `expression.threshold`.
#'   \item `pct_expr`: percentage of expressing cells.
#'   \item `avg_expr`: average expression across all cells in the group.
#'   \item `avg_expr_positive`: average expression among expressing cells only.
#' }
#'
#' The function returns both source-cluster-specific statistics and expression
#' summaries across all annotated groups. This makes it suitable for marker
#' inspection, heatmap visualization, dot-plot-style summaries, and radar plots.
#'
#' The function is compatible with Seurat v4 and v5 by attempting to retrieve
#' assay data through `layer` first and falling back to `slot` when needed.
#'
#' @param object A Seurat object.
#' @param group.by Character string. Metadata column used as cell-group
#'   annotation for marker detection. Default is `"seurat_annotations"`.
#' @param assay Character string or `NULL`. Assay used for marker detection and
#'   expression summarization. If `NULL`, [Seurat::DefaultAssay()] is used.
#'   Default is `NULL`.
#' @param slot Character string. Assay slot or layer used to calculate expression
#'   statistics. Common choices are `"data"` for normalized expression and
#'   `"counts"` for raw counts. Default is `"data"`.
#' @param top_n Integer. Number of top marker genes selected for each cell group.
#'   Default is `5`.
#' @param only.pos Logical. Whether to return only positive markers in
#'   [Seurat::FindAllMarkers()]. Default is `TRUE`.
#' @param min.pct Numeric. Minimum fraction of cells expressing a gene in either
#'   of the two groups tested by [Seurat::FindAllMarkers()]. Default is `0.25`.
#' @param logfc.threshold Numeric. Log-fold-change threshold used by
#'   [Seurat::FindAllMarkers()]. Default is `0.25`.
#' @param test.use Character string. Statistical test used by
#'   [Seurat::FindAllMarkers()]. Default is `"wilcox"`.
#' @param min.diff.pct Numeric. Minimum difference in detection percentage
#'   between the two groups compared by [Seurat::FindAllMarkers()]. Default is
#'   `-Inf`.
#' @param expression.threshold Numeric. Threshold used to define whether a gene
#'   is expressed in a cell when calculating `n_expr_cells` and `pct_expr`.
#'   Default is `0`.
#' @param exclude.mt Logical. Whether to remove mitochondrial genes from selected
#'   top markers. Human mitochondrial genes beginning with `"MT-"` and mouse
#'   mitochondrial genes beginning with `"mt-"` are removed. Default is `TRUE`.
#' @param exclude.ribo Logical. Whether to remove ribosomal protein genes
#'   beginning with `"RPS"`, `"RPL"`, `"Rps"`, or `"Rpl"`. Default is `FALSE`.
#' @param output_dir Character string or `NULL`. Optional directory for saving
#'   result tables as CSV files. If `NULL`, no files are written. Default is
#'   `NULL`.
#' @param file_prefix Character string. Prefix used for output CSV files when
#'   `output_dir` is not `NULL`. Default is `"CellWindX_top_markers"`.
#' @param seed Integer. Random seed set before marker detection. Default is
#'   `123`.
#' @param verbose Logical. Whether to print progress messages. Default is
#'   `TRUE`.
#'
#' @return A list with class `"CellWindX_TopMarkersStats"` containing:
#' \describe{
#'   \item{markers_all}{Complete marker table returned by
#'   [Seurat::FindAllMarkers()].}
#'   \item{top_markers}{Top marker genes selected for each source cell group.}
#'   \item{top_marker_stats}{Expression statistics of each selected marker gene
#'   within its source cell group.}
#'   \item{marker_expr_by_group}{Expression statistics of selected marker genes
#'   across all target cell groups. This table is used by downstream CellWindX
#'   heatmap and radar functions.}
#'   \item{top_gene_table}{A compact table listing top marker genes for each
#'   cell group.}
#'   \item{parameters}{A list of parameters used in the analysis.}
#' }
#'
#' @author Xiaofeng Yang, Chongqing Medical University
#'
#' @examples
#' counts <- matrix(
#'   c(
#'     25, 26, 24, 27, 25, 26, 24, 28, 25, 27,
#'      1,  2,  1,  1,  2,  1,  1,  2,  1,  1,
#'      1,  1,  2,  1,  1,  2,  1,  1,  2,  1,
#'
#'      1,  1,  2,  1,  1,  2,  1,  1,  2,  1,
#'     25, 27, 26, 24, 28, 25, 27, 26, 24, 28,
#'      1,  2,  1,  1,  2,  1,  1,  2,  1,  1,
#'
#'      1,  2,  1,  1,  2,  1,  1,  2,  1,  1,
#'      1,  1,  2,  1,  1,  2,  1,  1,  2,  1,
#'     25, 26, 28, 24, 27, 25, 26, 28, 24, 27,
#'
#'      5,  6,  5,  6,  5,  6,  5,  6,  5,  6,
#'      5,  6,  5,  6,  5,  6,  5,  6,  5,  6,
#'      5,  6,  5,  6,  5,  6,  5,  6,  5,  6
#'   ),
#'   nrow = 4,
#'   byrow = TRUE
#' )
#'
#' rownames(counts) <- c("CD3D", "MS4A1", "LYZ", "ACTB")
#' colnames(counts) <- paste0("Cell", seq_len(ncol(counts)))
#'
#' counts <- Matrix::Matrix(counts, sparse = TRUE)
#'
#' object <- Seurat::CreateSeuratObject(counts = counts)
#' object$cell_type <- rep(c("T cell", "B cell", "Myeloid"), each = 10)
#' object <- Seurat::NormalizeData(object, verbose = FALSE)
#'
#' res_marker <- CellWindX_TopMarkersStats(
#'   object = object,
#'   group.by = "cell_type",
#'   assay = "RNA",
#'   slot = "data",
#'   top_n = 1,
#'   only.pos = TRUE,
#'   min.pct = 0.10,
#'   logfc.threshold = 0,
#'   test.use = "t",
#'   verbose = FALSE
#' )
#'
#' head(res_marker$top_markers)
#' head(res_marker$top_marker_stats)
#' head(res_marker$marker_expr_by_group)
#' res_marker$top_gene_table
#'
#' @export
CellWindX_TopMarkersStats <- function(
    object,
    group.by = "seurat_annotations",
    assay = NULL,
    slot = "data",
    top_n = 5,
    only.pos = TRUE,
    min.pct = 0.25,
    logfc.threshold = 0.25,
    test.use = "wilcox",
    min.diff.pct = -Inf,
    expression.threshold = 0,
    exclude.mt = TRUE,
    exclude.ribo = FALSE,
    output_dir = NULL,
    file_prefix = "CellWindX_top_markers",
    seed = 123,
    verbose = TRUE
) {

  # -----------------------------
  # 1. Package check
  # -----------------------------
  if (!requireNamespace("Seurat", quietly = TRUE)) {
    stop("Package 'Seurat' is required.")
  }
  if (!requireNamespace("Matrix", quietly = TRUE)) {
    stop("Package 'Matrix' is required.")
  }

  # -----------------------------
  # 2. Basic checks
  # -----------------------------
  if (!inherits(object, "Seurat")) {
    stop("'object' must be a Seurat object.")
  }

  if (!group.by %in% colnames(object@meta.data)) {
    stop(
      paste0(
        "The metadata column '", group.by,
        "' was not found in object@meta.data."
      )
    )
  }

  if (is.null(assay)) {
    assay <- Seurat::DefaultAssay(object)
  }

  if (!assay %in% names(object@assays)) {
    stop(
      paste0(
        "Assay '", assay,
        "' was not found in the Seurat object."
      )
    )
  }

  top_n <- as.integer(top_n)

  if (top_n < 1) {
    stop("'top_n' must be >= 1.")
  }

  # -----------------------------
  # 3. Compatible GetAssayData
  # -----------------------------
  get_assay_matrix <- function(object, assay, slot) {

    mat <- tryCatch(
      {
        Seurat::GetAssayData(
          object = object,
          assay = assay,
          layer = slot
        )
      },
      error = function(e1) {
        Seurat::GetAssayData(
          object = object,
          assay = assay,
          slot = slot
        )
      }
    )

    return(mat)
  }

  # -----------------------------
  # 4. Prepare identities
  # -----------------------------
  set.seed(seed)

  old_ident <- Seurat::Idents(object)

  object@meta.data[[group.by]] <- as.factor(object@meta.data[[group.by]])
  Seurat::Idents(object) <- object@meta.data[[group.by]]

  group_levels <- levels(object@meta.data[[group.by]])

  if (verbose) {
    message("CellWindX: group.by = ", group.by)
    message("CellWindX: assay = ", assay)
    message("CellWindX: slot/layer = ", slot)
    message("CellWindX: number of groups = ", length(group_levels))
  }

  # -----------------------------
  # 5. Run FindAllMarkers
  # -----------------------------
  markers_all <- Seurat::FindAllMarkers(
    object = object,
    assay = assay,
    only.pos = only.pos,
    min.pct = min.pct,
    logfc.threshold = logfc.threshold,
    test.use = test.use,
    min.diff.pct = min.diff.pct,
    verbose = verbose
  )

  Seurat::Idents(object) <- old_ident

  if (nrow(markers_all) == 0) {
    stop("No marker genes were identified. Try lowering min.pct or logfc.threshold.")
  }

  # -----------------------------
  # 6. Standardize marker columns
  # -----------------------------
  if (!"gene" %in% colnames(markers_all)) {
    markers_all$gene <- rownames(markers_all)
  }

  fc_col <- NULL

  if ("avg_log2FC" %in% colnames(markers_all)) {
    fc_col <- "avg_log2FC"
  } else if ("avg_logFC" %in% colnames(markers_all)) {
    fc_col <- "avg_logFC"
  } else {
    stop("Cannot find avg_log2FC or avg_logFC column in FindAllMarkers output.")
  }

  if (!"cluster" %in% colnames(markers_all)) {
    stop("Cannot find 'cluster' column in FindAllMarkers output.")
  }

  markers_all$cluster <- as.character(markers_all$cluster)
  markers_all$gene <- as.character(markers_all$gene)

  # -----------------------------
  # 7. Optional gene filtering
  # -----------------------------
  markers_filtered <- markers_all

  if (exclude.mt) {
    markers_filtered <- markers_filtered[
      !grepl("^MT-", markers_filtered$gene) &
        !grepl("^mt-", markers_filtered$gene),
      ,
      drop = FALSE
    ]
  }

  if (exclude.ribo) {
    markers_filtered <- markers_filtered[
      !grepl("^RPS|^RPL|^Rps|^Rpl", markers_filtered$gene),
      ,
      drop = FALSE
    ]
  }

  if (nrow(markers_filtered) == 0) {
    stop("All markers were removed after filtering.")
  }

  # -----------------------------
  # 8. Select Top N markers per group
  # -----------------------------
  marker_split <- split(markers_filtered, markers_filtered$cluster)

  top_marker_list <- lapply(marker_split, function(df) {

    df <- df[order(
      df$p_val_adj,
      -df[[fc_col]],
      -df$pct.1
    ), , drop = FALSE]

    df <- df[seq_len(min(top_n, nrow(df))), , drop = FALSE]

    df$rank_in_cluster <- seq_len(nrow(df))

    return(df)
  })

  top_markers <- do.call(rbind, top_marker_list)
  rownames(top_markers) <- NULL

  useful_cols <- c(
    "cluster",
    "rank_in_cluster",
    "gene",
    "p_val",
    "p_val_adj",
    fc_col,
    "pct.1",
    "pct.2"
  )

  useful_cols <- useful_cols[useful_cols %in% colnames(top_markers)]
  top_markers <- top_markers[, c(useful_cols, setdiff(colnames(top_markers), useful_cols)), drop = FALSE]

  # -----------------------------
  # 9. Extract expression matrix
  # -----------------------------
  expr_mat <- get_assay_matrix(
    object = object,
    assay = assay,
    slot = slot
  )

  if (nrow(expr_mat) == 0 || ncol(expr_mat) == 0) {
    stop(
      paste0(
        "The selected assay slot/layer '", slot,
        "' is empty. Please run NormalizeData(object) first or use slot = 'counts'."
      )
    )
  }

  if (is.null(rownames(expr_mat)) || is.null(colnames(expr_mat))) {
    stop("Expression matrix must have rownames as genes and colnames as cells.")
  }

  # Keep only cells present in both meta.data and expression matrix
  common_cells <- intersect(rownames(object@meta.data), colnames(expr_mat))

  if (length(common_cells) == 0) {
    stop("No overlapping cell names between object@meta.data and expression matrix.")
  }

  meta_df <- object@meta.data[common_cells, , drop = FALSE]
  expr_mat <- expr_mat[, common_cells, drop = FALSE]

  meta_df[[group.by]] <- as.factor(meta_df[[group.by]])
  group_levels <- levels(meta_df[[group.by]])

  # Keep only top markers present in expression matrix
  top_markers$gene_in_matrix <- top_markers$gene %in% rownames(expr_mat)

  if (any(!top_markers$gene_in_matrix)) {
    warning(
      "Some selected marker genes were not found in the expression matrix and will be removed: ",
      paste(top_markers$gene[!top_markers$gene_in_matrix], collapse = ", ")
    )
  }

  top_markers <- top_markers[top_markers$gene_in_matrix, , drop = FALSE]

  if (nrow(top_markers) == 0) {
    stop("No selected top marker genes were found in the expression matrix.")
  }

  # -----------------------------
  # 10. Robust expression statistics
  # -----------------------------
  calc_gene_stats <- function(gene_vec, target_cells, expr_mat, threshold = 0) {

    gene_vec <- unique(as.character(gene_vec))
    target_cells <- unique(as.character(target_cells))

    gene_vec_valid <- intersect(gene_vec, rownames(expr_mat))
    target_cells_valid <- intersect(target_cells, colnames(expr_mat))

    if (length(gene_vec_valid) == 0 || length(target_cells_valid) == 0) {
      return(
        data.frame(
          gene = gene_vec,
          n_cells = length(target_cells_valid),
          n_expr_cells = NA_real_,
          pct_expr = NA_real_,
          avg_expr = NA_real_,
          avg_expr_positive = NA_real_,
          stringsAsFactors = FALSE
        )
      )
    }

    sub_mat <- expr_mat[gene_vec_valid, target_cells_valid, drop = FALSE]

    n_cells <- length(target_cells_valid)

    detected_mat <- sub_mat > threshold

    n_expr_cells <- Matrix::rowSums(detected_mat)
    pct_expr <- n_expr_cells / n_cells * 100
    avg_expr <- Matrix::rowMeans(sub_mat)

    avg_expr_positive <- rep(0, length(gene_vec_valid))
    names(avg_expr_positive) <- gene_vec_valid

    for (i in seq_along(gene_vec_valid)) {
      x <- as.numeric(sub_mat[i, ])
      x_pos <- x[x > threshold]

      if (length(x_pos) == 0) {
        avg_expr_positive[i] <- 0
      } else {
        avg_expr_positive[i] <- mean(x_pos)
      }
    }

    out <- data.frame(
      gene = gene_vec_valid,
      n_cells = n_cells,
      n_expr_cells = as.numeric(n_expr_cells),
      pct_expr = as.numeric(pct_expr),
      avg_expr = as.numeric(avg_expr),
      avg_expr_positive = as.numeric(avg_expr_positive),
      stringsAsFactors = FALSE
    )

    return(out)
  }

  # -----------------------------
  # 11. Marker stats in source cluster
  # -----------------------------
  top_marker_stats_list <- lapply(seq_len(nrow(top_markers)), function(i) {

    source_cluster <- as.character(top_markers$cluster[i])
    gene_i <- as.character(top_markers$gene[i])

    cells_i <- rownames(meta_df)[as.character(meta_df[[group.by]]) == source_cluster]

    stat_i <- calc_gene_stats(
      gene_vec = gene_i,
      target_cells = cells_i,
      expr_mat = expr_mat,
      threshold = expression.threshold
    )

    stat_i$source_cluster <- source_cluster
    stat_i$rank_in_cluster <- top_markers$rank_in_cluster[i]
    stat_i[[fc_col]] <- top_markers[[fc_col]][i]

    if ("p_val" %in% colnames(top_markers)) {
      stat_i$p_val <- top_markers$p_val[i]
    }

    if ("p_val_adj" %in% colnames(top_markers)) {
      stat_i$p_val_adj <- top_markers$p_val_adj[i]
    }

    if ("pct.1" %in% colnames(top_markers)) {
      stat_i$FindAllMarkers_pct1 <- top_markers$pct.1[i]
    }

    if ("pct.2" %in% colnames(top_markers)) {
      stat_i$FindAllMarkers_pct2 <- top_markers$pct.2[i]
    }

    return(stat_i)
  })

  top_marker_stats <- do.call(rbind, top_marker_stats_list)
  rownames(top_marker_stats) <- NULL

  top_marker_stats <- top_marker_stats[, c(
    "source_cluster",
    "rank_in_cluster",
    "gene",
    "n_cells",
    "n_expr_cells",
    "pct_expr",
    "avg_expr",
    "avg_expr_positive",
    fc_col,
    intersect(
      c("p_val", "p_val_adj", "FindAllMarkers_pct1", "FindAllMarkers_pct2"),
      colnames(top_marker_stats)
    )
  ), drop = FALSE]

  # -----------------------------
  # 12. Marker expression across all groups
  # -----------------------------
  all_group_stats_list <- list()
  idx <- 1

  for (source_cluster in unique(as.character(top_markers$cluster))) {

    genes_i <- unique(top_markers$gene[top_markers$cluster == source_cluster])

    for (target_cluster in group_levels) {

      cells_j <- rownames(meta_df)[as.character(meta_df[[group.by]]) == target_cluster]

      stat_j <- calc_gene_stats(
        gene_vec = genes_i,
        target_cells = cells_j,
        expr_mat = expr_mat,
        threshold = expression.threshold
      )

      stat_j$source_cluster <- source_cluster
      stat_j$target_cluster <- target_cluster

      all_group_stats_list[[idx]] <- stat_j
      idx <- idx + 1
    }
  }

  marker_expr_by_group <- do.call(rbind, all_group_stats_list)
  rownames(marker_expr_by_group) <- NULL

  marker_expr_by_group <- marker_expr_by_group[, c(
    "source_cluster",
    "target_cluster",
    "gene",
    "n_cells",
    "n_expr_cells",
    "pct_expr",
    "avg_expr",
    "avg_expr_positive"
  ), drop = FALSE]

  # -----------------------------
  # 13. Top gene list table
  # -----------------------------
  top_gene_list <- split(top_markers$gene, top_markers$cluster)

  top_gene_table <- data.frame(
    cluster = names(top_gene_list),
    top_genes = sapply(top_gene_list, function(x) paste(unique(x), collapse = ", ")),
    stringsAsFactors = FALSE
  )

  # -----------------------------
  # 14. Save files
  # -----------------------------
  if (!is.null(output_dir)) {

    if (!dir.exists(output_dir)) {
      dir.create(output_dir, recursive = TRUE)
    }

    write.csv(
      markers_all,
      file = file.path(output_dir, paste0(file_prefix, "_FindAllMarkers_all.csv")),
      row.names = FALSE
    )

    write.csv(
      top_markers,
      file = file.path(output_dir, paste0(file_prefix, "_top", top_n, "_markers.csv")),
      row.names = FALSE
    )

    write.csv(
      top_marker_stats,
      file = file.path(output_dir, paste0(file_prefix, "_top", top_n, "_stats_in_source_cluster.csv")),
      row.names = FALSE
    )

    write.csv(
      marker_expr_by_group,
      file = file.path(output_dir, paste0(file_prefix, "_top", top_n, "_expression_by_all_groups.csv")),
      row.names = FALSE
    )

    write.csv(
      top_gene_table,
      file = file.path(output_dir, paste0(file_prefix, "_top", top_n, "_gene_list.csv")),
      row.names = FALSE
    )
  }

  # -----------------------------
  # 15. Return
  # -----------------------------
  result <- list(
    markers_all = markers_all,
    top_markers = top_markers,
    top_marker_stats = top_marker_stats,
    marker_expr_by_group = marker_expr_by_group,
    top_gene_table = top_gene_table,
    parameters = list(
      group.by = group.by,
      assay = assay,
      slot = slot,
      top_n = top_n,
      only.pos = only.pos,
      min.pct = min.pct,
      logfc.threshold = logfc.threshold,
      test.use = test.use,
      expression.threshold = expression.threshold,
      exclude.mt = exclude.mt,
      exclude.ribo = exclude.ribo
    )
  )

  class(result) <- c("CellWindX_TopMarkersStats", class(result))

  return(result)
}

#' @importFrom stats ave median setNames
#' @importFrom utils write.csv
NULL

utils::globalVariables(c(
  "gene",
  "target_cluster",
  "pct_expr",
  "avg_expr",
  "axis_id",
  "x",
  "y",
  "grid_level",
  "xend",
  "yend",
  "label",
  "marker_id",
  "value"
))

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CellWindX_MarkerHeatmap.R
\name{CellWindX_MarkerHeatmap}
\alias{CellWindX_MarkerHeatmap}
\title{Draw marker-gene heatmap across annotated cell groups}
\usage{
CellWindX_MarkerHeatmap(
  marker_result,
  value_col = c("avg_expr", "pct_expr", "avg_expr_positive"),
  scale_method = c("zscore", "none"),
  plot_engine = c("complex", "ggplot"),
  palette = c("shanshui", "chongqing_modern", "girlish"),
  cluster_rows = FALSE,
  cluster_columns = FALSE,
  show_row_names = TRUE,
  show_column_names = TRUE,
  column_names_rot = 45,
  round_cell = TRUE,
  cell_width_mm = 5.5,
  cell_height_mm = 5.5,
  row_fontsize = 10,
  column_fontsize = 9,
  legend_position = "right",
  heatmap_title = NULL,
  zscore_clip = 2,
  output_file = NULL,
  output_width = 9,
  output_height = 5,
  dpi = 300,
  draw = TRUE
)
}
\arguments{
\item{marker_result}{A result object returned by \code{CellWindX_TopMarkersStats()},
or a data frame containing marker expression statistics.}

\item{value_col}{Character string. Value column used for heatmap visualization.
One of \code{"avg_expr"}, \code{"pct_expr"}, or \code{"avg_expr_positive"}.}

\item{scale_method}{Character string. Scaling method. One of \code{"zscore"} or \code{"none"}.}

\item{plot_engine}{Character string. Plotting engine. One of \code{"complex"} or \code{"ggplot"}.}

\item{palette}{Character string. Built-in CellWindX palette.}

\item{cluster_rows}{Logical. Whether to cluster rows.}

\item{cluster_columns}{Logical. Whether to cluster columns.}

\item{show_row_names}{Logical. Whether to show row names.}

\item{show_column_names}{Logical. Whether to show column names.}

\item{column_names_rot}{Numeric. Rotation angle of column names.}

\item{round_cell}{Logical. Whether to draw rounded cells when using ComplexHeatmap.}

\item{cell_width_mm}{Numeric. Cell width in millimeters.}

\item{cell_height_mm}{Numeric. Cell height in millimeters.}

\item{row_fontsize}{Numeric. Row name font size.}

\item{column_fontsize}{Numeric. Column name font size.}

\item{legend_position}{Character string. Legend position.}

\item{heatmap_title}{Character string or NULL. Heatmap title.}

\item{zscore_clip}{Numeric. Z-score clipping threshold.}

\item{output_file}{Character string or NULL. Optional output file path.}

\item{output_width}{Numeric. Output width in inches.}

\item{output_height}{Numeric. Output height in inches.}

\item{dpi}{Numeric. Output resolution.}

\item{draw}{Logical. Whether to draw the heatmap immediately when using ComplexHeatmap.}
}
\value{
A list with class \code{"CellWindX_MarkerHeatmap"}.
}
\description{
\code{CellWindX_MarkerHeatmap()} visualizes marker gene expression statistics
across annotated cell groups using either \code{ComplexHeatmap} or \code{ggplot2}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CellWindX_DimPlot.R
\name{CellWindX_DimPlot}
\alias{CellWindX_DimPlot}
\title{Visualize Seurat embeddings with CellWindX palettes}
\usage{
CellWindX_DimPlot(
  object,
  group.by = "seurat_annotations",
  reduction = c("umap", "tsne"),
  palette = c("shanshui", "chongqing_modern", "girlish"),
  label = TRUE,
  repel = TRUE,
  pt.size = 0.6,
  label.size = 4,
  alpha = 0.9,
  shuffle = TRUE,
  seed = 123,
  title = NULL,
  legend.position = "right"
)
}
\arguments{
\item{object}{A Seurat object containing dimensional reduction results.}

\item{group.by}{Character string. Metadata column used to color cells.
Default is \code{"seurat_annotations"}.}

\item{reduction}{Character string. Dimensional reduction to visualize.
One of \code{"umap"} or \code{"tsne"}.}

\item{palette}{Character string. Built-in CellWindX palette to use.
One of \code{"shanshui"}, \code{"chongqing_modern"}, or \code{"girlish"}.}

\item{label}{Logical. Whether to show group labels on the plot.
Default is \code{TRUE}.}

\item{repel}{Logical. Whether to repel text labels using ggrepel.
Default is \code{TRUE}.}

\item{pt.size}{Numeric. Point size passed to \code{\link[Seurat:DimPlot]{Seurat::DimPlot()}}.
Default is \code{0.6}.}

\item{label.size}{Numeric. Label font size. Reserved for future extension.
Default is \code{4}.}

\item{alpha}{Numeric or \code{NULL}. Point transparency. Values should usually
range from 0 to 1. Default is \code{0.9}.}

\item{shuffle}{Logical. Whether to randomly shuffle plotting order of cells.
Default is \code{TRUE}.}

\item{seed}{Integer. Random seed used when \code{shuffle = TRUE}.
Default is \code{123}.}

\item{title}{Character string or \code{NULL}. Plot title. If \code{NULL}, a default
title is automatically generated.}

\item{legend.position}{Character string. Legend position passed to
\code{\link[ggplot2:theme]{ggplot2::theme()}}. Common values include \code{"right"}, \code{"left"},
\code{"bottom"}, \code{"top"}, and \code{"none"}. Default is \code{"right"}.}
}
\value{
A ggplot object generated from \code{\link[Seurat:DimPlot]{Seurat::DimPlot()}}.
}
\description{
\code{CellWindX_DimPlot()} visualizes UMAP or t-SNE embeddings from a processed
Seurat object using built-in CellWindX color palettes. The function is designed
for annotated single-cell objects and supports three predefined aesthetic
styles: Chinese landscape-inspired, Chongqing modern, and girlish palettes.
}
\details{
This function is a wrapper around \code{\link[Seurat:DimPlot]{Seurat::DimPlot()}} with customized color
palettes and publication-oriented theme settings. It requires that the
selected dimensional reduction has already been computed and stored in
\code{object@reductions}, such as by running \code{\link[Seurat:RunUMAP]{Seurat::RunUMAP()}} or
\code{\link[Seurat:RunTSNE]{Seurat::RunTSNE()}}.

The built-in palettes support up to 20 annotated groups. If the number of
groups in \code{group.by} exceeds 20, the function will return an error.
}
\examples{
data("pbmc_small", package = "SeuratObject")

p1 <- CellWindX_DimPlot(
  object = pbmc_small,
  group.by = "groups",
  reduction = "tsne",
  palette = "shanshui"
)

p1

}
\author{
Xiaofeng Yang, Chongqing Medical University
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CellWindX_GeneRadar.R
\name{CellWindX_GeneRadar}
\alias{CellWindX_GeneRadar}
\title{Draw gene-level radar plots across annotated cell groups}
\usage{
CellWindX_GeneRadar(
  marker_result,
  genes,
  source_cluster = NULL,
  aggregate_fun = c("mean", "max", "first"),
  palette = c("shanshui", "chongqing_modern", "girlish"),
  scale_avg = TRUE,
  scale_pct = FALSE,
  pct_max = 100,
  avg_scale_method = c("max", "zscore", "none"),
  grid_levels = c(25, 50, 75, 100),
  axis_label_size = 5.2,
  grid_label_size = 3.2,
  facet_title_size = 12,
  line_width = 1.2,
  point_size = 2.8,
  fill_alpha = 0.18,
  facet = TRUE,
  ncol = NULL,
  show_points = TRUE,
  show_fill = TRUE,
  show_grid_label = TRUE,
  output_file = NULL,
  output_width = 11,
  output_height = 5.5,
  dpi = 300
)
}
\arguments{
\item{marker_result}{A result object returned by \code{CellWindX_TopMarkersStats()},
or a data frame containing marker expression statistics. If a data frame is
provided, it must contain the columns \code{source_cluster}, \code{target_cluster},
\code{gene}, \code{pct_expr}, and \code{avg_expr}.}

\item{genes}{Character vector. Gene symbols to visualize.}

\item{source_cluster}{Character vector or \code{NULL}. Optional source-cluster
filter. If provided, only marker statistics from the selected source
cluster(s) are used. Default is \code{NULL}.}

\item{aggregate_fun}{Character string. Method used to aggregate duplicated
gene-cell-group rows. One of \code{"mean"}, \code{"max"}, or \code{"first"}.
Default is \code{"mean"}.}

\item{palette}{Character string. Built-in CellWindX palette. One of
\code{"shanshui"}, \code{"chongqing_modern"}, or \code{"girlish"}.}

\item{scale_avg}{Logical. Whether to scale average expression values before
plotting. Default is \code{TRUE}.}

\item{scale_pct}{Logical. Whether to scale expression percentage values by
gene before plotting. If \code{FALSE}, the raw percentage values are used and
capped by \code{pct_max}. Default is \code{FALSE}.}

\item{pct_max}{Numeric. Maximum value used to cap percentage values when
\code{scale_pct = FALSE}. Default is \code{100}.}

\item{avg_scale_method}{Character string. Scaling method for average
expression. One of \code{"max"}, \code{"zscore"}, or \code{"none"}. \code{"max"} rescales
each gene to a 0-100 range by dividing by its maximum value. \code{"zscore"}
performs gene-wise Z-score scaling and rescales the result to 0-100.
\code{"none"} keeps the original average expression values. Default is \code{"max"}.}

\item{grid_levels}{Numeric vector. Radar grid levels. These values determine
the radius of the straight polygon grid lines. Default is
\code{c(25, 50, 75, 100)}.}

\item{axis_label_size}{Numeric. Font size of cell-group labels placed around
the radar plot. Default is \code{5.2}.}

\item{grid_label_size}{Numeric. Font size of radar grid labels. Default is
\code{3.2}.}

\item{facet_title_size}{Numeric. Font size of facet titles when multiple
genes are plotted. Default is \code{12}.}

\item{line_width}{Numeric. Width of radar polygon lines. Default is \code{1.2}.}

\item{point_size}{Numeric. Size of points on radar axes. Default is \code{2.8}.}

\item{fill_alpha}{Numeric. Transparency of polygon fill. Values should
usually range from 0 to 1. Default is \code{0.18}.}

\item{facet}{Logical. Whether to draw multiple genes as separate facets.
Default is \code{TRUE}.}

\item{ncol}{Integer or \code{NULL}. Number of columns used in facet layout when
\code{facet = TRUE}. Default is \code{NULL}.}

\item{show_points}{Logical. Whether to show points on radar axes.
Default is \code{TRUE}.}

\item{show_fill}{Logical. Whether to fill radar polygons. Default is \code{TRUE}.}

\item{show_grid_label}{Logical. Whether to show numeric grid labels.
Default is \code{TRUE}.}

\item{output_file}{Character string or \code{NULL}. Optional path used to save the
combined radar plot. Supported formats depend on \code{\link[ggplot2:ggsave]{ggplot2::ggsave()}},
such as \code{.pdf}, \code{.png}, \code{.tiff}, or \code{.svg}. Default is \code{NULL}.}

\item{output_width}{Numeric. Width of the saved plot in inches. Default is
\code{11}.}

\item{output_height}{Numeric. Height of the saved plot in inches. Default is
\code{5.5}.}

\item{dpi}{Numeric. Resolution used when saving raster formats. Default is
\code{300}.}
}
\value{
A list with class \code{"CellWindX_GeneRadar"} containing:
\describe{
\item{pct_plot}{A ggplot object showing expression percentage.}
\item{avg_plot}{A ggplot object showing average expression.}
\item{combined_plot}{A patchwork object combining percentage and average expression plots.}
\item{plot_data}{A data frame used for plotting.}
\item{pct_coord}{Radar coordinates for the expression percentage plot.}
\item{avg_coord}{Radar coordinates for the average expression plot.}
\item{palette}{The selected CellWindX palette.}
\item{genes}{Genes used for visualization.}
\item{parameters}{A list of function parameters used in the plot.}
}
}
\description{
\code{CellWindX_GeneRadar()} visualizes the expression pattern of selected genes
across annotated cell groups using straight-line spider radar plots. For each
selected gene, the function generates two radar plots: one for the percentage
of expressing cells and another for average expression.
}
\details{
This function is designed to directly accept the output generated by
\code{CellWindX_TopMarkersStats()}. It uses the \code{marker_expr_by_group} table from
that result object, which should contain gene expression statistics across
cell groups.

Each cell group is represented as one axis of the radar plot. Unlike polar
coordinate radar plots, this function manually calculates polygon coordinates,
resulting in straight polygon grid lines rather than curved circular grid
lines. This style is generally clearer for comparing cell-group-specific
expression patterns.

The function draws two panels:

\itemize{
\item \code{pct_plot}: expression percentage across cell groups.
\item \code{avg_plot}: average expression across cell groups.
}

The expression percentage and average expression panels use intentionally
distinct color sets within the selected CellWindX palette. Three built-in
palettes are available: \code{"shanshui"}, \code{"chongqing_modern"}, and \code{"girlish"}.
}
\examples{
marker_df <- data.frame(
  source_cluster = rep(c("T cell", "B cell"), each = 4),
  target_cluster = rep(c("T cell", "B cell", "Myeloid", "Platelet"), times = 2),
  gene = rep(c("CD3D", "MS4A1"), each = 4),
  pct_expr = c(92, 8, 15, 4, 6, 88, 12, 3),
  avg_expr = c(2.8, 0.2, 0.5, 0.1, 0.1, 2.5, 0.4, 0.1),
  stringsAsFactors = FALSE
)

radar_res <- CellWindX_GeneRadar(
  marker_result = marker_df,
  genes = c("CD3D", "MS4A1"),
  palette = "shanshui",
  scale_avg = TRUE,
  avg_scale_method = "max",
  facet = TRUE,
  ncol = 2
)

radar_res$combined_plot

}
\author{
Xiaofeng Yang, Chongqing Medical University
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CellWinX_TopMarkersStats.R
\name{CellWindX_TopMarkersStats}
\alias{CellWindX_TopMarkersStats}
\title{Identify top marker genes and summarize expression statistics}
\usage{
CellWindX_TopMarkersStats(
  object,
  group.by = "seurat_annotations",
  assay = NULL,
  slot = "data",
  top_n = 5,
  only.pos = TRUE,
  min.pct = 0.25,
  logfc.threshold = 0.25,
  test.use = "wilcox",
  min.diff.pct = -Inf,
  expression.threshold = 0,
  exclude.mt = TRUE,
  exclude.ribo = FALSE,
  output_dir = NULL,
  file_prefix = "CellWindX_top_markers",
  seed = 123,
  verbose = TRUE
)
}
\arguments{
\item{object}{A Seurat object.}

\item{group.by}{Character string. Metadata column used as cell-group
annotation for marker detection. Default is \code{"seurat_annotations"}.}

\item{assay}{Character string or \code{NULL}. Assay used for marker detection and
expression summarization. If \code{NULL}, \code{\link[Seurat:reexports]{Seurat::DefaultAssay()}} is used.
Default is \code{NULL}.}

\item{slot}{Character string. Assay slot or layer used to calculate expression
statistics. Common choices are \code{"data"} for normalized expression and
\code{"counts"} for raw counts. Default is \code{"data"}.}

\item{top_n}{Integer. Number of top marker genes selected for each cell group.
Default is \code{5}.}

\item{only.pos}{Logical. Whether to return only positive markers in
\code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}}. Default is \code{TRUE}.}

\item{min.pct}{Numeric. Minimum fraction of cells expressing a gene in either
of the two groups tested by \code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}}. Default is \code{0.25}.}

\item{logfc.threshold}{Numeric. Log-fold-change threshold used by
\code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}}. Default is \code{0.25}.}

\item{test.use}{Character string. Statistical test used by
\code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}}. Default is \code{"wilcox"}.}

\item{min.diff.pct}{Numeric. Minimum difference in detection percentage
between the two groups compared by \code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}}. Default is
\code{-Inf}.}

\item{expression.threshold}{Numeric. Threshold used to define whether a gene
is expressed in a cell when calculating \code{n_expr_cells} and \code{pct_expr}.
Default is \code{0}.}

\item{exclude.mt}{Logical. Whether to remove mitochondrial genes from selected
top markers. Human mitochondrial genes beginning with \code{"MT-"} and mouse
mitochondrial genes beginning with \code{"mt-"} are removed. Default is \code{TRUE}.}

\item{exclude.ribo}{Logical. Whether to remove ribosomal protein genes
beginning with \code{"RPS"}, \code{"RPL"}, \code{"Rps"}, or \code{"Rpl"}. Default is \code{FALSE}.}

\item{output_dir}{Character string or \code{NULL}. Optional directory for saving
result tables as CSV files. If \code{NULL}, no files are written. Default is
\code{NULL}.}

\item{file_prefix}{Character string. Prefix used for output CSV files when
\code{output_dir} is not \code{NULL}. Default is \code{"CellWindX_top_markers"}.}

\item{seed}{Integer. Random seed set before marker detection. Default is
\code{123}.}

\item{verbose}{Logical. Whether to print progress messages. Default is
\code{TRUE}.}
}
\value{
A list with class \code{"CellWindX_TopMarkersStats"} containing:
\describe{
\item{markers_all}{Complete marker table returned by
\code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}}.}
\item{top_markers}{Top marker genes selected for each source cell group.}
\item{top_marker_stats}{Expression statistics of each selected marker gene
within its source cell group.}
\item{marker_expr_by_group}{Expression statistics of selected marker genes
across all target cell groups. This table is used by downstream CellWindX
heatmap and radar functions.}
\item{top_gene_table}{A compact table listing top marker genes for each
cell group.}
\item{parameters}{A list of parameters used in the analysis.}
}
}
\description{
\code{CellWindX_TopMarkersStats()} identifies the top marker genes for each
annotated cell group in a Seurat object and summarizes their expression
percentage and average expression. The output is designed to be directly used
by downstream CellWindX visualization functions, including
\code{CellWindX_MarkerHeatmap()} and \code{CellWindX_GeneRadar()}.
}
\details{
This function first runs \code{\link[Seurat:FindAllMarkers]{Seurat::FindAllMarkers()}} using the cell-group
annotation specified by \code{group.by}. For each group, the top \code{N} marker genes
are selected according to adjusted P value, log-fold change, and detection
percentage.

For each selected marker gene, the function then calculates:

\itemize{
\item \code{n_cells}: number of cells in the target group.
\item \code{n_expr_cells}: number of cells with expression greater than
\code{expression.threshold}.
\item \code{pct_expr}: percentage of expressing cells.
\item \code{avg_expr}: average expression across all cells in the group.
\item \code{avg_expr_positive}: average expression among expressing cells only.
}

The function returns both source-cluster-specific statistics and expression
summaries across all annotated groups. This makes it suitable for marker
inspection, heatmap visualization, dot-plot-style summaries, and radar plots.

The function is compatible with Seurat v4 and v5 by attempting to retrieve
assay data through \code{layer} first and falling back to \code{slot} when needed.
}
\examples{
counts <- matrix(
  c(
    25, 26, 24, 27, 25, 26, 24, 28, 25, 27,
     1,  2,  1,  1,  2,  1,  1,  2,  1,  1,
     1,  1,  2,  1,  1,  2,  1,  1,  2,  1,

     1,  1,  2,  1,  1,  2,  1,  1,  2,  1,
    25, 27, 26, 24, 28, 25, 27, 26, 24, 28,
     1,  2,  1,  1,  2,  1,  1,  2,  1,  1,

     1,  2,  1,  1,  2,  1,  1,  2,  1,  1,
     1,  1,  2,  1,  1,  2,  1,  1,  2,  1,
    25, 26, 28, 24, 27, 25, 26, 28, 24, 27,

     5,  6,  5,  6,  5,  6,  5,  6,  5,  6,
     5,  6,  5,  6,  5,  6,  5,  6,  5,  6,
     5,  6,  5,  6,  5,  6,  5,  6,  5,  6
  ),
  nrow = 4,
  byrow = TRUE
)

rownames(counts) <- c("CD3D", "MS4A1", "LYZ", "ACTB")
colnames(counts) <- paste0("Cell", seq_len(ncol(counts)))

counts <- Matrix::Matrix(counts, sparse = TRUE)

object <- Seurat::CreateSeuratObject(counts = counts)
object$cell_type <- rep(c("T cell", "B cell", "Myeloid"), each = 10)
object <- Seurat::NormalizeData(object, verbose = FALSE)

res_marker <- CellWindX_TopMarkersStats(
  object = object,
  group.by = "cell_type",
  assay = "RNA",
  slot = "data",
  top_n = 1,
  only.pos = TRUE,
  min.pct = 0.10,
  logfc.threshold = 0,
  test.use = "t",
  verbose = FALSE
)

head(res_marker$top_markers)
head(res_marker$top_marker_stats)
head(res_marker$marker_expr_by_group)
res_marker$top_gene_table

}
\author{
Xiaofeng Yang, Chongqing Medical University
}
